@@ -27,9 +27,7 @@ import logging
 import re
 from collections import defaultdict
 
-from biolib.common import is_float, check_file_exists
-
-import dendropy
+from biolib.common import is_float, check_file_exists, canonical_gid
 
 """
 To do:
@@ -406,6 +404,7 @@ class Taxonomy(object):
                         check_species, 
                         check_group_names,
                         check_duplicate_names,
+                        check_capitalization,
                         report_errors=True):
         """Check if taxonomy forms a strict hierarchy with all expected ranks.
 
@@ -441,43 +440,53 @@ class Taxonomy(object):
         """
         
         # check for incomplete taxonomy strings or unexpected rank prefixes
-        invalid_ranks = {}
-        invalid_prefixes = {}
-        invalid_species_name = {}
-        invalid_group_name = {}
-        for taxon_id, taxa in taxonomy.items():
-            if check_ranks:
-                if len(taxa) != len(Taxonomy.rank_prefixes):
-                    invalid_ranks[taxon_id] = ';'.join(taxa)
-                    continue
+        try:
+            invalid_ranks = {}
+            invalid_prefixes = {}
+            invalid_species_name = {}
+            invalid_group_name = {}
+            invalid_capitalization = set()
+            for taxon_id, taxa in taxonomy.items():
+                if check_ranks:
+                    if len(taxa) != len(Taxonomy.rank_prefixes):
+                        invalid_ranks[taxon_id] = ';'.join(taxa)
+                        continue
 
-            if check_prefixes:
-                for r, taxon in enumerate(taxa):
-                    if taxon[0:3] != Taxonomy.rank_prefixes[r]:
-                        invalid_prefixes[taxon_id] = [taxon, ';'.join(taxa)]
-                        break
-                        
-            if check_group_names:
-                for taxon in taxa:
-                    canonical_taxon = ' '.join([t.strip() for t in re.split('_[A-Z]+(?= |$)', taxon[3:])]).strip()
-                    if canonical_taxon and re.match('^[a-zA-Z0-9- ]+$', canonical_taxon) is None:
-                        if not taxon.startswith('s__') or check_species:
-                            invalid_group_name[taxon_id] = [taxon, 'Taxon contains invalid characters']
+                if check_prefixes:
+                    for r, taxon in enumerate(taxa):
+                        if taxon[0:3] != Taxonomy.rank_prefixes[r]:
+                            invalid_prefixes[taxon_id] = [taxon, ';'.join(taxa)]
+                            break
+                            
+                if check_group_names:
+                    for taxon in taxa:
+                        canonical_taxon = ' '.join([t.strip() for t in re.split('_[A-Z]+(?= |$)', taxon[3:])]).strip()
+                        if canonical_taxon and re.match('^[a-zA-Z0-9- ]+$', canonical_taxon) is None:
+                            if not taxon.startswith('s__') or check_species:
+                                invalid_group_name[taxon_id] = [taxon, 'Taxon contains invalid characters']
 
-            if check_species:
-                genus_index = Taxonomy.rank_index['g__']
-                species_index = Taxonomy.rank_index['s__']
-                if len(taxa) > species_index:
-                    species_name = taxa[species_index]
-                    valid, error_msg = self.validate_species_name(species_name, require_full=True, require_prefix=True)
-                    if not valid:
-                        invalid_species_name[taxon_id] = [species_name, error_msg]
-                        
-                    if species_name != 's__':
-                        genus_name = taxa[genus_index]
-                        generic_name = species_name.split()[0]
-                        if genus_name[3:] != generic_name[3:]:
-                            invalid_species_name[taxon_id] = [species_name, 'Genus and generic names do not match: %s' % genus_name]
+                if check_species:
+                    genus_index = Taxonomy.rank_index['g__']
+                    species_index = Taxonomy.rank_index['s__']
+                    if len(taxa) > species_index:
+                        species_name = taxa[species_index]
+                        valid, error_msg = self.validate_species_name(species_name, require_full=True, require_prefix=True)
+                        if not valid:
+                            invalid_species_name[taxon_id] = [species_name, error_msg]
+                            
+                        if species_name != 's__':
+                            genus_name = taxa[genus_index]
+                            generic_name = species_name.split()[0]
+                            if genus_name[3:] != generic_name[3:]:
+                                invalid_species_name[taxon_id] = [species_name, 'Genus and generic names do not match: %s' % genus_name]
+                                
+                if check_capitalization:
+                    for taxon in taxa:
+                        if len(taxon) > 3 and taxon[3].islower():
+                            invalid_capitalization.add(taxon)
+        except:
+            self.logger.error('Exception raised while processing {} with the taxa set {}'.format(taxon_id, taxa))
+            raise
                         
         # check for duplicate names
         invalid_duplicate_name = []
@@ -546,8 +555,14 @@ class Taxonomy(object):
                 print('Taxonomy contains taxa with multiple parents:')
                 for child_taxon, parent_taxa in invalid_hierarchies.items():
                     print('%s\t%s' % (child_taxon, ', '.join(parent_taxa)))
+                    
+            if len(invalid_capitalization):
+                print('')
+                print('Taxa do not start with a capital letter:')
+                for taxon in invalid_capitalization:
+                    print('{}'.format(taxon))
 
-        return invalid_ranks, invalid_prefixes, invalid_species_name, invalid_hierarchies, invalid_group_name
+        return invalid_ranks, invalid_prefixes, invalid_species_name, invalid_hierarchies, invalid_group_name, invalid_capitalization
 
     def taxon_children(self, taxonomy):
         """Get children taxa for each taxonomic group.
@@ -742,6 +757,7 @@ class Taxonomy(object):
         """
 
         if isinstance(tree, str):
+            import dendropy
             tree = dendropy.Tree.get_from_path(tree, 
                                                 schema='newick', 
                                                 rooting="force-rooted", 
@@ -779,7 +795,7 @@ class Taxonomy(object):
             if len(taxa) == 7:
                 genus = taxa[5][3:]
                 species = taxa[6][3:]
-                if genus not in species:
+                if genus not in species and len(species.split()) == 1:
                     taxa[6] = 's__' + genus + ' ' + species
 
             taxa = self.fill_trailing_ranks(taxa)
@@ -787,7 +803,7 @@ class Taxonomy(object):
 
         return taxonomy
 
-    def read(self, taxonomy_file):
+    def read(self, taxonomy_file, use_canonical_gid = False):
         """Read Greengenes-style taxonomy file.
 
         Expected format is:
@@ -815,6 +831,9 @@ class Taxonomy(object):
                 line_split = line.split('\t')
                 unique_id = line_split[0]
                 
+                if use_canonical_gid:
+                    unique_id = canonical_gid(unique_id)
+                
                 tax_str = line_split[1].rstrip()
                 if tax_str[-1] == ';':
                     # remove trailing semicolons which sometimes
